import json
import pandas as pd
import time

#################################
#
#with open('logs.json', 'r') as data:
#	data = data.read()
#
#logs = json.loads(data)
#
########################

def get_data(file):

	with open(file, 'r') as data:
		data = data.read()

	logs = json.loads(data)

	#s = Sender('Test', '192.168.1.214')
	#logs = s.list_logs()


	df = pd.DataFrame(columns=['acquired_time'])


	lenghth = len(logs)
	i = 0
	while i < lenghth:
		for x in logs[i]:
			if x == "create_params_file:output":
				
				stats = logs[i][6]
				stats = stats.split(',')
				acquired_time = stats[3].split('"')
				acquired_time = acquired_time[3]
				print(acquired_time)

				df_temp = pd.DataFrame({'acquired_time': [acquired_time]})
				df = pd.concat([df, df_temp])
		i += 1
	
	df = df.reset_index(drop=True)
	return df



def wirte_logs_to_txt():
	f = open("logs.txt", "a+")
	
	lenghth = len(logs)
	i = 0
	while i < lenghth:
		for x in logs[i]:
			f.write(str(x))
			f.write('\r\n')
		f.write('#################################################### \r\n')
		i += 1
	
	f.close()

#df = get_data('logs.json')
#
#print(df)

def fibonacci(n):
    if n == 0:
        return 0
    elif n == 1:
        return 1
    return fibonacci(n - 1) + fibonacci(n - 2)

def memoize(func):
    cache = dict()

    def memoized_func(*args):
        if args in cache:
            return cache[args]
        result = func(*args)
        cache[args] = result
        return result

    return memoized_func

memoized_fibonacci = memoize(fibonacci)

x = memoized_fibonacci(10)
#x = fibonacci(35)
x = memoized_fibonacci(12)
#x = fibonacci(35)
print(x)


import setuptools
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rocklog",
    version="0.0.9",
    author="Thilo Schild",
    author_email="work@thilo-schild.de",
    description="collects Logs and presents them in a web app",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/thiloschild/RockLog",
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        ],
    packages=setuptools.find_packages(),
    install_requires=['pandas', 'dash', 'plotly', 'datetime'],
    python_requires='>=3.6',
    entry_points={

        'console_scripts': [
            'rocklog = rocklog.app:main'
        ],
        
    }
)


import os, sys; sys.path.append(os.path.dirname(os.path.realpath(__file__)))
import json
import pandas as pd
from urllib.request import Request, urlopen
from urllib.error import URLError
import json
from pathlib import Path
import socket
from collections import namedtuple

#########################################################
#class

class Sender(object):
    def __init__(self,
                 name,
                 host='192.168.1.214',
                 port=8745, 
                 encoding="cp1251"):
        self.host = host
        self.port = port
        self.encoding = encoding
        self.name = name
        self.ip = socket.gethostbyname(socket.gethostname())
        self.project_id = self.__get_project_id()
        self.group = name

    def __sock(self, route, message=None):
        url = f"http://{self.host}:{self.port}/{route}"
        request = Request(url)
        request.add_header('Content-Type', 'application/json; charset=utf-8')
        if message is None:
            message = json.dumps(self.name).encode(self.encoding)
        return urlopen(request, message)

    def __get_project_id(self):
        with self.__sock('get_project_id') as s:
            return json.loads(s.read())

    def log(self, key, value):

        _log = json.dumps((self.ip, 
                           self.project_id,
                           self.name,
                           self.group,
                           key, 
                           dump2json(value))).encode(self.encoding)
        with self.__sock('log', _log) as s:
            return json.loads(s.read())            

    def update_group(self, group):
        self.group = dump2json(group) # general.

    def list_logs(self):

        LOG = namedtuple('log', 'date ip project_id group process_name key value')
        with self.__sock('get_all_logs') as s:
            return [LOG(*log) for log in json.loads(s.read())]


#####################################################################
#functions

def get_data_mock(file):

    with open(file, 'r') as data:
        data = data.read()

    logs = json.loads(data)

    return logs

def get_data():

    s = Sender('Test', '192.168.1.214')
    logs = s.list_logs()

    return logs

def __get_df():

    logs = get_data_mock('logs.json')
    #logs = get_data()

    df = pd.DataFrame(columns=['id',
                               'name',
                               'sample',
                               'aquired',
                               'proteins',
                               'peptides',
                               'queries',
                               'hits',
                               'type',
                               'analyzed',
                               'msResolution',
                               'ChromFWHM_Min'])
 
    lenghth = len(logs)
    i = 0
    f1 = False
    f2 = False
    while i < lenghth:
        for x in logs[i]:

            if x == "create_params_file:output":
                stats = logs[i][6]
                stats = stats.split(',')
                aquired = stats[3].split('"')
                aquired = aquired[3]
                msResolution = stats[10].split(':')
                msResolution = float(msResolution[2])
                ChromFWHM_Min = stats[7].split(':')
                ChromFWHM_Min = float(ChromFWHM_Min[2])
                f1 = True
            if x == "get_search_stats:output":
                #date
                analyzed = logs[i][0].split('.', 2)
                analyzed = analyzed[0]
                #experiment number
                id = logs[i][2]
                type = logs[i][3]
                #peptides_cnt + proteins_cnt + hits_cnt + queries_cnt
                search_stats = logs[i][6]
                stats = search_stats.split(',', 7)
                peptides = stats[5].split(':', 2)
                peptides = int(peptides[1])
                proteins = stats[6].split(':', 2)
                proteins = proteins[1].split('}',2)
                proteins = int(proteins[0])
                queries = stats[3].split(':', 2)
                queries = int(queries[1])
                hits = stats[4].split(':', 2)
                hits = int(hits[1])
                #acquired_name + sample_description
                name = stats[1].split(':', 2)
                name = name[1].split('"', 3)
                name = name[1]
                sample = stats[2].split(':', 2)
                sample = sample[1].split('"', 3)
                sample = sample[1]
                f2 = True

        if (f1==True) and (f2==True):
            df_temp = pd.DataFrame({'id': [id],
                                    'name': [name],
                                    'sample': [sample],
                                    'aquired': [aquired],
                                    'proteins': [proteins],
                                    'peptides': [peptides],
                                    'queries': [queries],
                                    'hits': [hits],
                                    'type': [type],
                                    'analyzed': [analyzed],
                                    'msResolution': [msResolution],
                                    'ChromFWHM_Min': [ChromFWHM_Min]})
            df = pd.concat([df, df_temp])
            f1=False
            f2=False

        i += 1
    
    df = df.reset_index(drop=True)
    return df

class refresh_cache(object):
    def __init__(self, foo):
        self.output = None
        self.foo = foo

    def __call__(self, refresh=False, *args, **kwds):
        if self.output is None or refresh:
            self.output = self.foo(*args, **kwds)
        return self.output

mem_get_df = refresh_cache(__get_df)

def sort_df(df):
    df = df.sort_values(by=['analyzed'])
    df = df.reset_index(drop=True)
    return df

def get_index_list(df):
    index_list = list(df.index.values)
    return index_list

def split_filter_part(filter_part):

    operators = [['ge ', '>='],
                ['le ', '<='],
                ['lt ', '<'],
                ['gt ', '>'],
                ['ne ', '!='],
                ['eq ', '='],
                ['contains '],
                ['datestartswith ']]
    for operator_type in operators:
        for operator in operator_type:
            if operator in filter_part:
                name_part, value_part = filter_part.split(operator, 1)
                name = name_part[name_part.find('{') + 1: name_part.rfind('}')]

                value_part = value_part.strip()
                v0 = value_part[0]
                if (v0 == value_part[-1] and v0 in ("'", '"', '`')):
                    value = value_part[1: -1].replace('\\' + v0, v0)
                else:
                    try:
                        value = float(value_part)
                    except ValueError:
                        value = value_part

                # word operators need spaces after them in the filter string,
                # but we don't want these later
                return name, operator_type[0].strip(), value

    return [None] * 3

def filter_data(df, filter):

    filtering_expressions = filter.split(' && ')
    dff = df
    for filter_part in filtering_expressions:
        col_name, operator, filter_value = split_filter_part(filter_part)

        if operator in ('eq', 'ne', 'lt', 'le', 'gt', 'ge'):
            # these operators match pandas series operator method names
            dff = dff.loc[getattr(dff[col_name], operator)(filter_value)]
        elif operator == 'contains':
            dff = dff.loc[dff[col_name].str.contains(filter_value)]
        elif operator == 'datestartswith':
            # this is a simplification of the front-end filtering logic,
            # only works with complete fields in standard format
            dff = dff.loc[dff[col_name].str.startswith(filter_value)]

    return dff

def get_page_count(df, PAGE_SIZE):

    lenght_df = len(df.index)
    PAGE_COUNT = int(lenght_df/PAGE_SIZE)+1

    return PAGE_COUNT



import os, sys; sys.path.append(os.path.dirname(os.path.realpath(__file__)))
import dash
import dash_table
import dash_core_components as dcc
import dash_html_components as html
from dash.dependencies import Input, Output
from dash.exceptions import PreventUpdate
import plotly.express as px
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import pandas as pd
import urllib.parse
from datetime import datetime
#############
from functions import *


#################################################################################
#Dash
#################################################################################

df = mem_get_df()
df = sort_df(df)
now = datetime.now()

external_stylesheets = ['https://codepen.io/chriddyp/pen/bWLwgP.css']

app = dash.Dash(__name__, external_stylesheets=external_stylesheets)

##################################################################################
#Layout
app.layout = html.Div(children=[
    html.H1(children='Logs from The Rock'),

    html.Div(children='''
        
    '''),

    html.A(html.Button('Refresh', id='refresh_button', n_clicks=0), href='/'),
    html.Div(id='refreshed', children=''),

    dcc.Graph(id='graph'),


    dcc.Dropdown(id='dropdown',
                 options=[
                   {'label': 'msResolution', 'value': 'msResolution'},
                   {'label': 'ChromFWHM_Min', 'value': 'ChromFWHM_Min'}
                 ],
                 value='msResolution'
        ),

    dcc.Graph(id='info_graph',
        config={'displayModeBar': False},
    ),

    html.Br(),

    dash_table.DataTable(
        id='table-sorting-filtering',
        columns=[
            {'name': 'id', 'id': 'id', 'type': 'numeric'},
            {'name': 'name', 'id': 'name', 'type': 'text'},
            {'name': 'sample', 'id': 'sample', 'type': 'text'},
            {'name': 'aquired', 'id': 'aquired', 'type': 'datetime'},
            {'name': 'proteins', 'id': 'proteins', 'type': 'numeric'},
            {'name': 'peptides', 'id': 'peptides', 'type': 'numeric'},
            {'name': 'queries', 'id': 'queries', 'type': 'numeric'},
            {'name': 'hits', 'id': 'hits', 'type': 'numeric'},
            {'name': 'type', 'id': 'type', 'type': 'text'},
            {'name': 'analyzed', 'id': 'analyzed', 'type': 'datetime'},
            {'name': 'msResolution', 'id': 'msResolution', 'type': 'numeric'},
            {'name': 'ChromFWHM_Min', 'id': 'ChromFWHM_Min', 'type': 'numeric'},
            
        ],
        
        page_current=0,
        page_size=40,
        page_action='custom',

        filter_action='custom',
        filter_query='',

        sort_action='custom',
        sort_mode='single',
        sort_by=[]),
    
    html.Br(),
    'Page size: ',
    dcc.Input(
        id='datatable-page-size',
        type='number',
        min=1,
        value=40
    ),
    html.Br(),html.Br(),

    html.Div(html.A(html.Button('Download'),
        id='test',
        download="logs.csv",
        href=""),
    id='download-link'),

    html.Br(),

], style={'marginLeft': 25, 'marginRight': 25, 'marginTop': 15, 'marginBottom': 15})

##################################################################################
#App Callback

#First Graph
@app.callback(
    Output('graph', 'figure'),
    [Input('table-sorting-filtering', 'filter_query')])
def update_figure(filter):
    df = mem_get_df()
    dff = filter_data(df, filter)

    id = dff["id"]
    proteins = dff["proteins"]
    peptides = dff["peptides"]
    analyzed = dff["analyzed"]
    index_list = get_index_list(df)

    fig=go.Figure(
        data=[
            go.Bar(name="Proteins", x=index_list, y=proteins, yaxis='y', offsetgroup=1,
                   text=['<b>%s</b><br>id: %d<br>Proteins: %d<br>Peptides: %d<br>Queries: %d<br>Hits: %d<br>Sample: %s<br>aquired: %s<br>analyzed: %s<br>type: %s<br>msResolution: %s<br>ChromFWHM_Min: %s'%(t,s,r,v,w,q,x,y,z,a,b,c) 
                   for t,s,r,v,w,q,x,y,z,a,b,c in dff.loc[:,['name','id','proteins','peptides','queries','hits','sample','aquired','analyzed','type','msResolution','ChromFWHM_Min']].values], hoverinfo = 'text',),
            go.Bar(name='Peptides', x=index_list, y=peptides, yaxis='y2', offsetgroup=2,
                   text=['<b>%s</b><br>id: %d<br>Proteins: %d<br>Peptides: %d<br>Queries: %d<br>Hits: %d<br>Sample: %s<br>aquired: %s<br>analyzed: %s<br>type: %s<br>msResolution: %s<br>ChromFWHM_Min: %s'%(t,s,r,v,w,q,x,y,z,a,b,c) 
                   for t,s,r,v,w,q,x,y,z,a,b,c in dff.loc[:,['name','id','proteins','peptides','queries','hits','sample','aquired','analyzed','type','msResolution','ChromFWHM_Min']].values], hoverinfo = 'text',),
        ],
        layout=
            {
             'plot_bgcolor': '#FFF',
             'yaxis': {'title': 'Proteins'},
             'yaxis2': {'title': 'Peptides', 'overlaying': 'y', 'side': 'right'},
             'xaxis': {'title': 'Experiments', 'showticklabels': False}
            }
        )

    fig.update_layout(
        barmode="group",
        hoverlabel=dict(
        bgcolor="white", 
        font_size=16,
        font_family="Rockwell"
        )
    )
    return fig

#Second Graph
@app.callback(
    Output('info_graph', 'figure'),
    [Input('dropdown', 'value'),
     Input('table-sorting-filtering', 'filter_query')])
def update_info_graph(selected_dropdown_value, filter):
    df = mem_get_df()
    dff = filter_data(df, filter)
    dff = sort_df(dff)
    selected_dropdown_value = dff[selected_dropdown_value]

    fig = px.line(dff, x=get_index_list(dff), y=selected_dropdown_value)

    fig.update_layout(xaxis_title="Experiments")

    return fig

#Table
@app.callback(
    Output('table-sorting-filtering', 'data'),
    [Input('table-sorting-filtering', 'page_current'),
     Input('table-sorting-filtering', 'page_size'),
     Input('table-sorting-filtering', 'sort_by'),
     Input('table-sorting-filtering', 'filter_query')])
def update_table(page_current, page_size, sort_by, filter):
    df = mem_get_df()
    dff = filter_data(df, filter)
    if len(sort_by):
        dff = dff.sort_values(
            [col['column_id'] for col in sort_by],
            ascending=[
                col['direction'] == 'asc'
                for col in sort_by
            ],
            inplace=False
        )

    page = page_current
    size = page_size
    return dff.iloc[page * size: (page + 1) * size].to_dict('records')

#Table - update page count
@app.callback(
    Output('table-sorting-filtering', 'page_count'),
    [Input('table-sorting-filtering', 'page_current'),
     Input('table-sorting-filtering', 'page_size'),
     Input('table-sorting-filtering', 'sort_by'),
     Input('table-sorting-filtering', 'filter_query')])
def update_table_page_count(page_current, page_size, sort_by, filter):
    dff = filter_data(df, filter)
    page = page_current
    size = page_size
    page_count = get_page_count(dff, size)
    return page_count

#Table - update page size
@app.callback(
    Output('table-sorting-filtering', 'page_size'),
    [Input('datatable-page-size', 'value')])
def update_table_page_size(page_size):
    if page_size != None:
        return page_size
    else:
        return 1

#Download
@app.callback(
    dash.dependencies.Output('test', 'href'),
    [dash.dependencies.Input('download-link', 'n_clicks')])
def update_download_link(n_clicks):
    df = mem_get_df(refresh=True)
    csv_string = df.to_csv(index=False, encoding='utf-8')
    csv_string = csv_string.replace(" ", "_")
    csv_string = "data:text/csv;charset=utf-8," + urllib.parse.quote(csv_string)
    return csv_string

#Refresh Button
@app.callback(
    dash.dependencies.Output('refreshed', 'children'),
    [dash.dependencies.Input('refresh_button', 'n_clicks')])
def update_download_link(n_clicks):
    df = mem_get_df(refresh=True)
    now = datetime.now()
    msg = 'Refreshed on: {}'.format(now.strftime("%d/%m/%Y %H:%M:%S"))
    return msg
##################################################################################
#run the app

def main():
    app.run_server(debug=True, host='0.0.0.0', port=8050)

if __name__ == '__main__':
    main()